@@ -2,26 +2,30 @@
 # coding: utf-8
 
 from abc import ABC, abstractmethod
-from napoleontoolbox.features import features_type
-import sys
-
+from napoleontoolbox.file_saver import dropbox_file_saver
 import pandas as pd
 
 import numpy as np
 
 import torch
 
+from napoleontoolbox.features import features_type
+from os import walk
+import pandas as pd
+from pathlib import Path
 
 class AbstractAssembler(ABC):
-    def __init__(self, features_path, returns_path, root='../data/', user = 'napoleon', features_saving_path = '_features.npy', features_names_saving_path = '_features_names.npy'):
-        super().__init__()
-        self.root =  root
-        self.user =  user
-        self.features_path = features_path
-        self.returns_path = returns_path
-        self.features_saving_path = features_saving_path
-        self.features_names_saving_path = features_names_saving_path
 
+    def __init__(self, drop_token=None, returns_pkl_file_name='returns.pkl', features_pkl_file_name='features.pkl', features_saving_suffix='_features.npy', features_names_saving_suffix='_features_names.npy', local_root_directory='../data/', user='napoleon',macro_features_directory = 'features_macro'):
+        super().__init__()
+        self.local_root_directory = local_root_directory
+        self.user = user
+        self.features_pkl_file_name = features_pkl_file_name
+        self.features_saving_suffix = features_saving_suffix
+        self.features_names_saving_suffix = features_names_saving_suffix
+        self.returns_pkl_file_name = returns_pkl_file_name
+        self.macro_features_directory = macro_features_directory
+        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token)
 
     @abstractmethod
     def assembleFeature(self, feature_type, n_past_features):
@@ -29,23 +33,29 @@ class AbstractAssembler(ABC):
 
 
 class FeaturesAssembler(AbstractAssembler):
-    def reassembleAdvancedFeaturesForClusterization(self, normalize = True, stationarize = True, clustering_size = 21, simple = False, features_extraction = True):
-        df = pd.read_pickle(self.root + self.returns_path)
 
-        df['Date'] = pd.to_datetime(df['Date'])
-        df = df.set_index('Date')
-        strats = [col for col in list(df.columns) if col != 'Date']
-
-        df = df.fillna(method='ffill')
-
-        advanced_features = pd.read_pickle(self.root + self.features_path)
-        features_names = [col for col in list(advanced_features.columns) if col!='Date']
+    def reassembleAdvancedFeaturesForClusterization(self, feature_type, n_past_features = 21, simple = True):
+        if (feature_type is features_type.FeaturesType.HISTORY or feature_type is features_type.FeaturesType.HISTORY_ADVANCED) and n_past_features is None:
+            return
+        if (feature_type is features_type.FeaturesType.STANDARD or feature_type is features_type.FeaturesType.STANDARD_ADVANCED) and n_past_features is not None:
+            return
+        print('feature_type' + str(feature_type))
+        print('n_past_features' + str(n_past_features))
 
 
-        advanced_features['Date'] = pd.to_datetime(advanced_features['Date'])
+        np.random.seed(0)
+        torch.manual_seed(0)
+
+        advanced_features, advanced_features_names = self.preprocessFeatures()
+        macro_features, macro_features_names = self.preprocessMacroFeatures()
+        df, strats, strat_features_names = self.preprocessReturns()
+
+        duplicated_features = [my_feat for my_feat in advanced_features_names if my_feat in macro_features]
+        advanced_features = advanced_features.drop(columns=duplicated_features)
+        advanced_features_names = [col for col in list(advanced_features.columns) if col != 'Date']
+
+        features_names = advanced_features_names + macro_features_names
 
-        # quotes_df=quotes_df.sort_values(by='date', ascending=True)
-        # quotes_df.head()
 
         print(df.columns)
         print(df.shape)
@@ -55,45 +65,17 @@ class FeaturesAssembler(AbstractAssembler):
 
         # Computationnal period (default 1 year)
 
-        np.random.seed(0)
-        torch.manual_seed(0)
 
         ##===================##
         ##  Setting targets  ##
         ##===================##
 
-        #
-        df_bis = df.copy()
-        # df_ret = df_bis.pct_change().fillna(0.)
-        # ret = df_ret.values
 
         print('merging')
-        df_bis = pd.merge(df_bis, advanced_features, how='left', on=['Date'])
+        df_ret = pd.merge(df, advanced_features, how='left', on=['Date'])
 
-        print('merging done')
-        df_bis.index = df_bis['Date']
-        df_bis = df_bis.drop(columns=['Date'])
-        print('return')
-
-        df_bis = df_bis.fillna(method='ffill').fillna(method='bfill')
-        df_ret = df_bis.copy()
-
-        print('computing returns')
-        for col in strats:
-            print(col + str(len(df_bis.columns)))
-            df_ret[col] = df_bis[col].pct_change().fillna(0.)
-
-        if stationarize:
-            print('stationarizing features')
-            for col in features_names:
-                print(col + str(len(df_bis.columns)))
-                df_ret[col] = df_bis[col].pct_change().replace([np.inf, -np.inf], np.nan).fillna(0.)
-
-        if normalize:
-            print('normalizing features') # careful : forward looking bias
-            for col in features_names:
-                print(col + str(len(df_bis.columns)))
-                df_ret[col] = (df_ret[col] - df_ret[col].mean()) / df_ret[col].std(ddof=0)
+        df_ret = df_ret.replace([np.inf, -np.inf], np.nan)
+        df_ret=df_ret.fillna(0.)
 
         print('number of infs in advanced features')
         print(np.isinf(df_ret[features_names]).sum(axis=0))
@@ -108,69 +90,92 @@ class FeaturesAssembler(AbstractAssembler):
         print(np.isnan(df_ret[strats]).sum(axis=0))
 
         if simple:
-            return strats, features_names, df_ret
+            return strats, strat_features_names, features_names, df_ret
+
 
         ret_df = df_ret[strats]
         ret = ret_df.values
 
-        feat = df_ret[features_names].values
+        feat = df_ret[strat_features_names + features_names].values
 
         T = df.index.size
-        N = df.columns.size
+        N = len(strats)
 
-        if features_extraction:
-            features = np.zeros([T, 6*len(features_names)], np.float32)
-        else:
-            features = np.zeros([T, clustering_size, len(features_names)], np.float32)
+        all_features_length = len(features_names) + len(strat_features_names)
+
+        if feature_type is features_type.FeaturesType.HISTORY_ADVANCED:
+            features = np.zeros([T, n_past_features, N + all_features_length], np.float32)
+            predictor_names = strats  + features_names + strat_features_names
+
+        if feature_type is features_type.FeaturesType.HISTORY:
+            features = np.zeros([T, n_past_features, N], np.float32)
 
 
-        # for t in range(max(n_past_features, s), T - s):
-        for t in range(clustering_size, T):
-            np.random.seed(0)
-            torch.manual_seed(0)
-            # the output to predict cannot be computed in the future
-            # we still assemble the predictors
-            # Set input data
-            t_n = min(max(t - clustering_size, 0), T)
-            F = feat[t_n: t, :]
-            X_back = ret[t_n: t, :]
+        if feature_type is features_type.FeaturesType.HISTORY or feature_type is features_type.FeaturesType.HISTORY_ADVANCED:
+            for t in range(n_past_features, T):
+                np.random.seed(0)
+                torch.manual_seed(0)
+                # the output to predict cannot be computed in the future
+                # we still assemble the predictors
+                # Set input data
+                t_n = min(max(t - n_past_features, 0), T)
+                F = feat[t_n: t, :]
+                X_back = ret[t_n: t, :]
 
-            if features_extraction:
-                F_mean = np.mean(F, axis=0)
-                F_quant_zero = np.quantile(F, 0., axis=0)
-                F_quant_twenty_five = np.quantile(F, 0.25, axis=0)
-                F_quant_fifty = np.quantile(F, 0.5, axis=0)
-                F_quant_seventy_five = np.quantile(F, 0.75, axis=0)
-                F_quant_hundred = np.quantile(F, 1., axis=0)
-                features[t: t + 1] = np.concatenate(
-                    (F_mean, F_quant_zero, F_quant_twenty_five, F_quant_fifty, F_quant_seventy_five, F_quant_hundred),
-                    axis=0)
+                if feature_type is features_type.FeaturesType.HISTORY_ADVANCED or feature_type is features_type.FeaturesType.HISTORY:
+                    if feature_type is features_type.FeaturesType.HISTORY_ADVANCED:
+                        X_back = np.concatenate((X_back, F), axis=1)
+                        predictor_names = features_names + strat_features_names
+                    if feature_type is features_type.FeaturesType.HISTORY:
+                        predictor_names = strats
+
+                    features[t: t + 1] = X_back
+
+                if t % 500 == 0:
+                    print('{:.2%}'.format(t / T))
+                # we compute the utility output to predict only if not in future
+        else :
+            if feature_type is features_type.FeaturesType.STANDARD_ADVANCED:
+
+                #features = np.zeros([T, N + all_features_length], np.float32)
+                features = feat
+                predictor_names =  features_names + strat_features_names
+
+            if feature_type is features_type.FeaturesType.STANDARD:
+                #features = np.zeros([T, N], np.float32)
+                features = ret
+                predictor_names = strats
+
+        return  predictor_names, features
+
+
+
+    def preprocessMacroFeatures(self):
+        macro_features_path = Path(self.local_root_directory+self.macro_features_directory)
+        f = []
+        for (dirpath, dirnames, filenames) in walk(macro_features_path):
+            f.extend(filenames)
+        all_merged_df = None
+        for my_file in f:
+            unpickled_df = pd.read_pickle(str(macro_features_path / my_file))
+            unpickled_df.columns = [col.replace(' Index last quote', '').replace(' ', '_') for col in unpickled_df.columns]
+            if all_merged_df is None:
+                all_merged_df = unpickled_df
             else:
-                features[t: t + 1] = F
+                all_merged_df = pd.merge(all_merged_df, unpickled_df, how='inner', left_index=True, right_index=True)
 
-            if t % 500 == 0:
-                print('{:.2%}'.format(t / T))
-            # we compute the utility output to predict only if not in future
-
-        print('saved files')
-        print('number of nan/infinity features')
-        print(np.isnan(features).sum(axis=0).sum())
-        print(np.isinf(features).sum(axis=0).sum())
-        if np.isnan(features).sum(axis=0).sum() > 0:
-            raise Exception('nan values for assembled features')
-        if np.isinf(features).sum(axis=0).sum() > 0:
-            raise Exception('inf values for assembled features')
-        return strats, features_names, features
+        all_merged_df['Date'] = all_merged_df.index
+        all_merged_df['Date'] = pd.to_datetime(all_merged_df['Date'])
+        macro_features = [col for col in list(all_merged_df.columns) if col!='Date']
+        return all_merged_df, macro_features
 
 
-    def preprocessFeature(self):
+    def preprocessFeatures(self):
 
-        features = pd.read_pickle(self.root + self.features_path)
+        features = self.dbx.local_overwrite_and_load_pickle( folder='', subfolder='', returns_pkl_file_name=self.features_pkl_file_name, local_root_directory = self.local_root_directory)
         features_names = [col for col in list(features.columns) if col!='Date']
-
         features['Date'] = pd.to_datetime(features['Date'])
 
-
         # MM 3 mois
         for col in features_names:
             features["MM63_{}".format(col)] = features[col] / features[col].shift(1).rolling(window=63).mean() - 1
@@ -241,19 +246,12 @@ class FeaturesAssembler(AbstractAssembler):
 
     def preprocessReturns(self):
 
-        df = pd.read_pickle(self.root + self.returns_path)
+        df = self.dbx.local_overwrite_and_load_pickle( folder='', subfolder='', returns_pkl_file_name=self.returns_pkl_file_name, local_root_directory = self.local_root_directory)
         df['Date'] = pd.to_datetime(df['Date'])
         strats = [col for col in list(df.columns) if col != 'Date']
-
-        #@todo later
-        #corr_strats = [col1+'_'+col2 for col1 in strats for col2 in strats]
-
         df = df.set_index('Date')
-
-
         df = df.fillna(method='ffill').fillna(method='bfill')
 
-
         # MM 3 mois
         for col in strats:
             df["MM63_{}".format(col)] = df[col] / df[col].shift(1).rolling(window=63).mean() - 1
@@ -320,7 +318,10 @@ class FeaturesAssembler(AbstractAssembler):
 
     def assembleFeature(self, feature_type, n_past_features):
 
-        if feature_type is not features_type.FeaturesType.HISTORY and n_past_features is not None:
+        if (feature_type is features_type.FeaturesType.HISTORY or feature_type is features_type.FeaturesType.HISTORY_ADVANCED) and n_past_features is None:
+            return
+
+        if (feature_type is features_type.FeaturesType.STANDARD or feature_type is features_type.FeaturesType.STANDARD_ADVANCED) and n_past_features is not None:
             return
 
         print('feature_type' + str(feature_type))
@@ -330,14 +331,22 @@ class FeaturesAssembler(AbstractAssembler):
         np.random.seed(0)
         torch.manual_seed(0)
 
-        advanced_features, features_names = self.preprocessFeature()
-        df, strats, strat_features_names = self.preprocessReturns()
+        advanced_features, advanced_features_names = self.preprocessFeatures()
+        macro_features, macro_features_names = self.preprocessMacroFeatures()
+        df_ret, strats, strat_features_names = self.preprocessReturns()
+
+        duplicated_features = [my_feat for my_feat in advanced_features_names if my_feat in macro_features]
+        advanced_features = advanced_features.drop(columns=duplicated_features)
+        advanced_features_names = [col for col in list(advanced_features.columns) if col!='Date']
+
+        features_names = advanced_features_names + macro_features_names
+
 
         # quotes_df=quotes_df.sort_values(by='date', ascending=True)
         # quotes_df.head()
 
-        print(df.columns)
-        print(df.shape)
+        print(df_ret.columns)
+        print(df_ret.shape)
 
         print(advanced_features.columns)
         print(advanced_features.shape)
@@ -351,9 +360,14 @@ class FeaturesAssembler(AbstractAssembler):
 
 
         print('merging')
-        df_ret = pd.merge(df, advanced_features, how='left', on=['Date'])
+        df_ret = pd.merge(df_ret, advanced_features, how='left', on=['Date'])
+        df_ret = pd.merge(df_ret, macro_features, how='left', on=['Date'], right_index=True)
 
+        df_ret = df_ret.replace([np.inf, -np.inf], np.nan)
+        df_ret=df_ret.fillna(0.)
 
+        print('number of infs in macro features')
+        print(np.isinf(df_ret[macro_features_names]).sum(axis=0))
 
         print('number of infs in advanced features')
         print(np.isinf(df_ret[features_names]).sum(axis=0))
@@ -364,6 +378,9 @@ class FeaturesAssembler(AbstractAssembler):
         print('number of nans in advanced features')
         print(np.isnan(df_ret[features_names]).sum(axis=0))
 
+        print('number of nans in macro features')
+        print(np.isnan(df_ret[macro_features_names]).sum(axis=0))
+
         print('number of nans in returns')
         print(np.isnan(df_ret[strats]).sum(axis=0))
 
@@ -373,8 +390,8 @@ class FeaturesAssembler(AbstractAssembler):
 
         feat = df_ret[strat_features_names + features_names].values
 
-        T = df.index.size
-        N = df.columns.size
+        T = df_ret.index.size
+        N = len(strats)
 
         all_features_length = len(features_names) + len(strat_features_names)
 
@@ -400,21 +417,12 @@ class FeaturesAssembler(AbstractAssembler):
                 if feature_type is features_type.FeaturesType.HISTORY_ADVANCED or feature_type is features_type.FeaturesType.HISTORY:
                     if feature_type is features_type.FeaturesType.HISTORY_ADVANCED:
                         X_back = np.concatenate((X_back, F), axis=1)
-                    features[t: t + 1] = X_back
-                elif (feature_type is features_type.FeaturesType.STANDARD or feature_type is features_type.FeaturesType.STANDARD_ADVANCED):
-                    F_mean = np.nanmean(F, axis=0)
-                    std = np.std(X_back, axis=0).reshape([N, 1])
-                    mean = np.mean(X_back, axis=0)
-                    mat_std = std @ std.T
-                    mat_std[mat_std == 0.] = 1.
-                    mat_corr = np.cov(X_back, rowvar=False) / mat_std
+                        predictor_names = features_names + strat_features_names
+                    if feature_type is features_type.FeaturesType.HISTORY:
+                        predictor_names = strats
 
-                    if (feature_type is features_type.FeaturesType.STANDARD_ADVANCED):
-                        features[t: t + 1] = np.transpose(
-                            np.concatenate((mean, mat_corr.flatten(), F_mean), axis=0))
-                    if (feature_type is features_type.FeaturesType.STANDARD):
-                        features[t: t + 1] = np.transpose(
-                            np.concatenate((mean, mat_corr.flatten()), axis=0))
+
+                    features[t: t + 1] = X_back
 
                 if t % 500 == 0:
                     print('{:.2%}'.format(t / T))
@@ -424,7 +432,7 @@ class FeaturesAssembler(AbstractAssembler):
 
                 #features = np.zeros([T, N + all_features_length], np.float32)
                 features = feat
-                predictor_names = strats + features_names + strat_features_names
+                predictor_names =  features_names + strat_features_names
 
             if feature_type is features_type.FeaturesType.STANDARD:
                 #features = np.zeros([T, N], np.float32)
@@ -440,18 +448,6 @@ class FeaturesAssembler(AbstractAssembler):
         if np.isinf(features).sum(axis=0).sum() > 0:
             raise Exception('inf values for assembled features')
 
-        print('saving file')
-        features_saving_path = self.root  + self.user + '_' + feature_type.name + '_' + str(n_past_features) + self.features_saving_path
-        print(features_saving_path)
-        np.save(features_saving_path, features)
-
-        features_names_saving_path = self.root  + self.user + '_' + feature_type.name + '_' + str(n_past_features) + self.features_names_saving_path
-        print(features_names_saving_path)
-        np.save(features_names_saving_path, predictor_names)
-
-
-
-
-
+        self.dbx.local_features_npy_save_and_upload(features = features,features_names= predictor_names,feature_type=feature_type, n_past_features = n_past_features , local_root_directory= self.local_root_directory, user = self.user, features_saving_suffix=self.features_saving_suffix, features_names_saving_suffix=self.features_names_saving_suffix)
 
 
