@@ -8,23 +8,24 @@ import pandas as pd
 import numpy as np
 
 from napoleontoolbox.utility import metrics
+from napoleontoolbox.file_saver import dropbox_file_saver
 from scipy.optimize import Bounds, LinearConstraint, minimize
 
 import torch
 
 
 class AbstractAssembler(ABC):
-    def __init__(self, features_path, returns_path, root='../data/', user = 'napoleon', low_bound = 0.02, up_bound = 0.4, supervision_saving_path='_supervision.npy'):
-
+    def __init__(self, drop_token=None, features_pkl_file_name='features.pkl', returns_pkl_file_name='returns.pkl', local_root_directory='../data/', user='napoleon',
+                 supervision_npy_file_suffix='_supervision.npy', low_bound=0.02, up_bound=0.4):
         super().__init__()
-        self.root =  root
-        self.user =  user
-        self.features_path = features_path
-        self.returns_path = returns_path
+        self.features_pkl_file_name = features_pkl_file_name
+        self.returns_pkl_file_name = returns_pkl_file_name
+        self.local_root_directory = local_root_directory
+        self.user = user
+        self.supervision_npy_file_suffix = supervision_npy_file_suffix
+        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token)
         self.low_bound = low_bound
         self.up_bound = up_bound
-        self.supervision_saving_path = supervision_saving_path
-
 
     @abstractmethod
     def computeUtility(self, s):
@@ -33,40 +34,25 @@ class AbstractAssembler(ABC):
 
 class UtilitySuperviser(AbstractAssembler):
     def computeUtility(self, s):
-        print('quotes loading')
-
-        df = pd.read_pickle(self.root + self.returns_path)
-
+        df = self.dbx.local_overwrite_and_load_pickle( folder='', subfolder='', returns_pkl_file_name=self.returns_pkl_file_name, local_root_directory = self.local_root_directory)
         df['Date'] = pd.to_datetime(df['Date'])
         df = df.set_index('Date')
         strats = [col for col in list(df.columns) if col != 'Date']
         df = df.fillna(method='ffill')
-
-        print('advanced features loading')
-        advanced_features = pd.read_pickle(self.root + self.features_path)
+        print('size return '+str(df.shape))
+        advanced_features = self.dbx.local_overwrite_and_load_pickle( folder='', subfolder='', returns_pkl_file_name=self.features_pkl_file_name, local_root_directory = self.local_root_directory)
         features_names = [col for col in list(advanced_features.columns) if col!='Date']
         advanced_features['Date'] = pd.to_datetime(advanced_features['Date'])
 
-
-        # quotes_df=quotes_df.sort_values(by='date', ascending=True)
-        # quotes_df.head()
-
-        print(df.columns)
-        print(df.shape)
-
-        # Computationnal period (default 1 year)
+        print('size advanced_features '+str(advanced_features.shape))
 
         np.random.seed(0)
         torch.manual_seed(0)
-
         ##===================##
         ##  Setting targets  ##
         ##===================##
 
-        #
         df_bis = df.copy()
-        # df_ret = df_bis.pct_change().fillna(0.)
-        # ret = df_ret.values
 
         print('merging')
         df_bis = pd.merge(df_bis, advanced_features, how='left', on=['Date'])
@@ -100,21 +86,22 @@ class UtilitySuperviser(AbstractAssembler):
         N = df.columns.size
         w0 = np.ones([N]) / N
         w_ = w0
-        previous_w_ = w_
+        previous_w_dic = {}
+        previous_w_dic['f_minVar']   = w_.copy()
+        previous_w_dic['f_maxMean']  = w_.copy()
+        previous_w_dic['f_MeanVar']  = w_.copy()
+        previous_w_dic['f_sharpe']   = w_.copy()
+        previous_w_dic['f_calmar']   = w_.copy()
+        previous_w_dic['f_drawdown'] = w_.copy()
 
         # print('saving df_ret')
         # df_ret.to_pickle('../data/df_ret.pkl')
 
         print("recomputing supervision weights")
         # Set constraints and minimze
-        const_sum = LinearConstraint(np.ones([1, N]), [1], [1])
-        const_ind = Bounds(self.low_bound * np.ones([N]), self.up_bound * np.ones([N]))
-
-        # features = np.zeros([T , N * N + N+ len(features_names)], np.float32)
-        # features = np.zeros([T, N * N], np.float32)
 
         utility_size = 6
-        result = np.zeros([T, N, utility_size], np.float32)
+        result = np.zeros([T, N, utility_size], np.float64)
 
         def process(series):
             # True if less than 50% of obs. are constant
@@ -164,16 +151,14 @@ class UtilitySuperviser(AbstractAssembler):
                     # Search optimal weights => target
                     f_list = [f_minVar, f_maxMean, f_sharpe, f_MeanVar, f_calmar, f_drawdown]
                     i = 0
-                    previous_w_ = w_
-                    w__ = w_[assets]
                     for f in f_list:
+                        previous_w_ = previous_w_dic[f.__name__]
                         np.random.seed(0)
                         torch.manual_seed(0)
                         # Optimize f
                         w__ = minimize(
                             f,
-                            # w0[assets],
-                            w__,
+                            previous_w_[assets],
                             method='SLSQP',
                             constraints=[const_sum],
                             bounds=const_ind
@@ -187,24 +172,25 @@ class UtilitySuperviser(AbstractAssembler):
                         w_[~assets] = 0.
                         s_w = w_.sum()
 
-                        # Verify if sum(w) = 1 and set output
                         if s_w == 1.:
-                            result[t: t + 1, :, i] = w_
-
+                            next_w = w_
                         elif s_w != 0.:
-                            result[t: t + 1, :, i] = w_ / s_w
-
+                            next_w = w_ / s_w
                         else:
-                            result[t: t + 1, :, i] = w0
+                            next_w = w0
+                        result[t: t + 1, :, i] = next_w
+                        previous_w_dic[f.__name__] = next_w
                         i += 1
+
+
                 else:
                     for ii in range(utility_size):
                         result[t: t + 1, :, ii] = w0
 
-        print('saving file')
-        np.save(self.root + self.user + '_' + str(s) + self.supervision_saving_path, result)
-        print('saved files')
-        print('number of nan/infinity output')
+
+        print('saving localy file')
+
+
         print(np.isnan(result).sum(axis=0).sum())
         print(np.isinf(result).sum(axis=0).sum())
         if np.isnan(result).sum(axis=0).sum() > 0:
@@ -212,6 +198,12 @@ class UtilitySuperviser(AbstractAssembler):
         if np.isinf(result).sum(axis=0).sum() > 0:
             raise Exception('trouble : nan in supervised output')
 
+        self.dbx.local_supervision_npy_save_and_upload(data = result, rebal = s , local_root_directory= self.local_root_directory, user = self.user, supervision_npy_file_suffix= self.supervision_npy_file_suffix)
+
+        print('saved files')
+        print('number of nan/infinity output')
+
+
 
 
 
