@@ -2,58 +2,21 @@
 # coding: utf-8
 
 
-""" Basis of rolling models.
-Examples
---------
-# >>> roll_xgb = RollingXGB(X, y)
-# >>> for pred_eval, pred_test in roll_xgb(256, 64):
-# >>>     plot(pred_eval, pred_test)
-"""
-
-# Built-in packages
-
-# External packages
-import numpy as np
-from matplotlib import pyplot as plt
-import torch
-
-# Local packages
-# from fynance.models.xgb import XGBData
-
-from napoleontoolbox.backtest.dynamic_plot_backtest import DynaPlotBackTest
-
-import shap
-
-# Set plot style
-plt.style.use('seaborn')
-
-
-__all__ = ['_RollingBasis', 'RollMultiLayerPerceptron']
-
-#!/usr/bin/env python3
-# coding: utf-8
-
-
-""" Basis of neural networks models. """
-
-# Built-in packages
-
-# External packages
-import numpy as np
 import pandas as pd
 import torch
 import torch.nn
 import numpy as np
 import torch.nn.functional as F
+import shap
+import pickle
 
 from sklearn.metrics import mean_squared_error
 
-# Local packages
+from napoleontoolbox.roller import roller
+from napoleontoolbox.callback import ts_callback
+from functools import partial
 
-__all__ = ['BaseNeuralNet', 'MultiLayerPerceptron']
-
-
-class BaseNeuralNet(torch.nn.Module):
+class _PerceptronBaseNeuralNet(torch.nn.Module):
     """ Base object for neural network model with PyTorch.
     Inherits of torch.nn.Module object with some higher level methods.
     Attributes
@@ -79,7 +42,7 @@ class BaseNeuralNet(torch.nn.Module):
         """ Initialize. """
         torch.nn.Module.__init__(self)
 
-    def set_optimizer(self, criterion, optimizer, **kwargs):
+    def set_optimizer(self, criterion, optimizer, lr_type, **kwargs):
         """ Set the optimizer object.
         Set optimizer object with specified `criterion` as loss function and
         any `kwargs` as optional parameters.
@@ -101,7 +64,16 @@ class BaseNeuralNet(torch.nn.Module):
         """
         self.criterion = criterion()
         self.optimizer = optimizer(self.parameters(), **kwargs)
+        self.lr = None
+        lr = kwargs['lr']
 
+        sched_standard = ts_callback.create_standard_annealing_profile(lr)
+        cbf_func = partial(ts_callback.ParamScheduler, 'lr', sched_standard,lr_type)
+
+        cb = cbf_func()
+        setattr(self, cb.name, cb)
+        cb.set_runner(self)
+        self.cbs.append(cb)
         return self
 
     @torch.enable_grad()
@@ -121,8 +93,8 @@ class BaseNeuralNet(torch.nn.Module):
         loss = self.criterion(outputs, y)
         loss.backward()
         self.optimizer.step()
-
-        return loss
+        self.loss = loss.item()
+        return loss.item()
 
     @torch.no_grad()
     def predict(self, X):
@@ -183,7 +155,7 @@ class BaseNeuralNet(torch.nn.Module):
             raise ValueError('Unkwnown data type: {}'.format(type(X)))
 
 
-class MultiLayerPerceptron(BaseNeuralNet):
+class MultiLayerPerceptron(_PerceptronBaseNeuralNet):
     r""" Neural network with MultiLayer Perceptron architecture.
     Refered as vanilla neural network model, with `n` hidden layers s.t
     n :math:`\geq` 1, with each one a specified number of neurons.
@@ -223,7 +195,7 @@ class MultiLayerPerceptron(BaseNeuralNet):
     def __init__(self, X, y, layers=[], activation=None, drop=None,
                  x_type=None, y_type=None, bias=True, activation_kwargs={}):
         """ Initialize object. """
-        BaseNeuralNet.__init__(self)
+        _PerceptronBaseNeuralNet.__init__(self)
 
         self.set_data(X=X, y=y, x_type=x_type, y_type=y_type)
         layers_list = []
@@ -261,11 +233,13 @@ class MultiLayerPerceptron(BaseNeuralNet):
     def forward(self, x):
         """ Forward computation. """
         x = self.drop(x)
-        for name, layer in enumerate(self.layers):
-            if name == (self.n_layers) and self.activation.__name__ == F.relu.__name__:
-                x = layer(x)
-            else:
-                x = self.activation(layer(x))
+        for layer in self.layers:
+            x = self.activation(layer(x))
+        # for name, layer in enumerate(self.layers):
+        #     if name == (self.n_layers) and self.activation.__name__ == F.relu.__name__:
+        #         x = layer(x)
+        #     else:
+        #         x = self.activation(layer(x))
 
         return x
 
@@ -298,247 +272,10 @@ def _type_convert(dtype):
     else:
         raise ValueError('Unkwnown type: {}'.format(str(dtype)))
 
-class _RollingBasis:
-    """ Base object to roll a neural network model.
-    Rolling over a time axis with a train period from `t - n` to `t` and a
-    testing period from `t` to `t + s`.
-    Parameters
-    ----------
-    X, y : array_like
-        Respectively input and output data.
-    f : callable, optional
-        Function to transform target, e.g. ``torch.sign`` function.
-    index : array_like, optional
-        Time index of data.
-    Methods
-    -------
-    __call__
-    Attributes
-    ----------
-    n, s, r : int
-        Respectively size of training, testing and rolling period.
-    b, e, T : int
-        Respectively batch size, number of epochs and size of entire dataset.
-    t : int
-        The current time period.
-    y_eval, y_test : np.ndarray[ndim=1 or 2, dtype=np.float64]
-        Respectively evaluating (or training) and testing predictions.
-    """
-
-    # TODO : other methods
-    def __init__(self, X, y, f=None, index=None):
-        """ Initialize shape of target. """
-        self.T = X.shape[0]
-        self.y_shape = y.shape
-
-        if f is None:
-            self.f = lambda x: x
-
-        else:
-            self.f = f
-
-        if index is None:
-            self.idx = np.arange(self.T)
-
-        else:
-            self.idx = index
-
-    # TODO : fix callable method to overwritten problem with torch.nn.Module
-    def __call__(self, train_period, test_period, start=0, end=None,  epochs=1):
-        """ Callable method to set target features data, and model.
-        Parameters
-        ----------
-        train_period, test_period : int
-            Size of respectively training and testing sub-periods.
-        start : int, optional
-            Starting observation, default is first observation.
-        end : int, optional
-            Ending observation, default is last observation.
-        roll_period : int, optional
-            Size of the rolling period, default is the same size of the
-            testing sub-period.
-        eval_period : int, optional
-            Size of the evaluating period, default is the same size of the
-            testing sub-period if training sub-period is large enough.
-        batch_size : int, optional
-            Size of a training batch, default is 64.
-        epochs : int, optional
-            Number of epochs on the same subperiod, default is 1.
-        Returns
-        -------
-        _RollingBasis
-            The rolling basis model.
-        """
-        # Set size of subperiods
-        self.n = train_period
-        self.s = test_period
-        self.e = epochs
-
-        # Set boundary of period
-        self.T = self.T if end is None else min(self.T, end)
-        self.t = max(self.n - self.s, start)
-
-        return self
-
-    def __iter__(self):
-        """ Set iterative method. """
-        self.y_eval = np.zeros(self.y_shape)
-        self.y_test = np.zeros(self.y_shape)
-        self.loss_train = []
-        self.loss_eval = []
-        self.loss_test = []
-
-        return self
-
-    def __next__(self):
-        """ Incrementing method. """
-        # TODO : to finish
-        # Time forward incrementation
-        self.t += self.s
-
-        if self.t > (self.T-1):
-            raise StopIteration
-
-        if self.t + self.s > self.T:
-            # output to train would need the future : we do not retrain the networl
-            return slice(self.t - self.n, self.t), slice(self.t, self.T)
-
-        # TODO : Set training part in an other method
-        # Run epochs
-        for epoch in range(self.e):
-            train_slice = slice(self.t - self.n, self.t-self.s)
-            lo = self._train(
-                X=self.X[train_slice],
-                y=self.f(self.y[train_slice]),
-            )
-            self.loss_train += [lo.item()]
-        # Set eval and test periods
-        return slice(self.t - self.n, self.t), slice(self.t, self.t + self.s)
-
-    def run(self, backtest_plot=True, backtest_kpi=True, figsize=(9, 6)):
-        """ Run neural network model.
-        Parameters
-        ----------
-        backtest_plot : bool, optional
-            If True, display plot of backtest performances.
-        backtest_kpi : bool, optional
-            If True, display kpi of backtest performances.
-        """
-        perf_eval = 100. * np.ones(self.y.shape)
-        perf_test = 100. * np.ones(self.y.shape)
-        # Set dynamic plot object
-        f, (ax_1, ax_2) = plt.subplots(2, 1, figsize=figsize)
-        plt.ion()
-        ax_loss = DynaPlotBackTest(
-            f, ax_1, title='Model loss', ylabel='Loss', xlabel='Epochs',
-            yscale='log', tick_params={'axis': 'x', 'labelsize': 10}
-        )
-        ax_perf = DynaPlotBackTest(
-            f, ax_2, title='Model perf.', ylabel='Perf.',
-            xlabel='Date', yscale='log',
-            tick_params={'axis': 'x', 'rotation': 30, 'labelsize': 10}
-        )
-
-        # TODO : get stats, loss, etc.
-        # TODO : plot loss, perf, etc.
-        for eval_slice, test_slice in self:
-            # Predict on training and testing period
-            self.y_eval[eval_slice] = self.sub_predict(self.X[eval_slice])
-            self.y_test[test_slice] = self.sub_predict(self.X[test_slice])
-            # Compute losses
-            self.loss_eval += [self.criterion(
-                torch.from_numpy(self.y_eval[eval_slice]),
-                self.y[eval_slice]
-            ).item()]
-            self.loss_test += [self.criterion(
-                torch.from_numpy(self.y_test[test_slice]),
-                self.y[test_slice]
-            ).item()]
-
-            if backtest_kpi:
-                # Display %
-                pct = self.t - self.n - self.s
-                pct = pct / (self.T - self.n - self.T % self.s)
-                txt = '{:5.2%} is done | '.format(pct)
-                txt += 'Eval loss is {:5.2} | '.format(self.loss_eval[-1])
-                txt += 'Test loss is {:5.2} | '.format(self.loss_test[-1])
-                print(txt, end='\r')
-
-            if backtest_plot:
-                # Set performances of training period
-                returns = np.sign(self.y_eval[eval_slice]) * self.y[eval_slice].numpy()
-                cumret = np.exp(np.cumsum(returns, axis=0))
-                perf_eval[eval_slice] = perf_eval[self.t - self.r - 1] * cumret
-
-                # Set performances of estimated period
-                returns = np.sign(self.y_test[test_slice]) * self.y[test_slice].numpy()
-                cumret = np.exp(np.cumsum(returns, axis=0))
-                perf_test[test_slice] = perf_test[self.t - 1] * cumret
-
-                ax_loss.ax.clear()
-                ax_perf.ax.clear()
-                # Plot loss
-                ax_loss.plot(np.array([self.loss_test]).T, names='Test',
-                             col='BuGn', lw=2.)
-                ax_loss.plot(
-                    np.array([self.loss_eval]).T, names='Eval', col='YlOrBr',
-                    loc='upper right', ncol=2, fontsize=10, handlelength=0.8,
-                    columnspacing=0.5, frameon=True, lw=1.,
-                )
-
-                # Plot perf
-                ax_perf.plot(
-                    perf_test[: self.t + self.s],
-                    x=self.idx[: self.t + self.s],
-                    names='Test set', col='GnBu', lw=1.7, unit='perf',
-                )
-                ax_perf.plot(
-                    perf_eval[: self.t], x=self.idx[: self.t],
-                    names='Eval set', col='OrRd', lw=1.2, unit='perf'
-                )
-                ax_perf.ax.legend(loc='upper left', fontsize=10, frameon=True,
-                                  handlelength=0.8, ncol=2, columnspacing=0.5)
-                f.canvas.draw()
-                # plt.draw()
-
-        return self
 
 
-class RollingXGB(_RollingBasis):
-    """ Rolling version of eXtrem Gradient Boosting model.
-    Model will roll train and test periods over a time axis, at time `t` the
-    training period is from `t - n` to `t` and the testing period from `t` to
-    `t + s`.
-    Attributes
-    ----------
-    n, s : int
-        Respectively size of training and testing period.
-    """
 
-    # TODO : to finish
-    def __init__(self, X, y, **kwargs):
-        """ Set data to XGBoot model.
-        Parameters
-        ----------
-        X, y : np.ndarray[ndim=2, dtype=np.float64]
-            Respectively features with shape `(T, N)` and target with shape
-            `(T, 1)` of the model.
-        kwargs : dict, optional
-            Parameters of DMatrix object, cf XGBoost documentation [1]_.
-        References
-        ----------
-        .. [1] https://boosted_trees.readthedocs.io/en/latest/python/python_api.html
-        """
-        _RollingBasis.__init__(self, X, y)
-        # self.data = XGBData(X, label=y, **kwargs)
-        self.bst = None
-
-    def _train(self):
-        # self.bst = xgb.train(params, )
-        pass
-
-
-class RollMultiLayerPerceptron(MultiLayerPerceptron, _RollingBasis):
+class RollMultiLayerPerceptron(MultiLayerPerceptron, roller._RollingBasis):
     """ Rolling version of the vanilla neural network model.
     TODO:
     - fix train and predict methods
@@ -549,13 +286,14 @@ class RollMultiLayerPerceptron(MultiLayerPerceptron, _RollingBasis):
     def __init__(self, X, y, layers=[], activation=None, drop=None, bias=True,
                  x_type=None, y_type=None, activation_kwargs={}, **kwargs):
         """ Initialize rolling multi-layer perceptron model. """
-        _RollingBasis.__init__(self, X, y, **kwargs)
+        roller._RollingBasis.__init__(self, X, y, **kwargs)
         MultiLayerPerceptron.__init__(self, X, y, layers=layers, bias=bias,
                                       activation=activation, drop=drop,
                                       x_type=x_type, y_type=y_type,
                                       activation_kwargs=activation_kwargs)
 
-    def set_roll_period(self, train_period, test_period, start=0, end=None,epochs=1):
+
+    def set_roll_period(self, train_period, test_period, start=0, end=None,repass_steps=1):
         """ Callable method to set target features data, and model.
         Parameters
         ----------
@@ -580,9 +318,9 @@ class RollMultiLayerPerceptron(MultiLayerPerceptron, _RollingBasis):
         _RollingBasis
             The rolling basis model.
         """
-        return _RollingBasis.__call__(
+        return roller._RollingBasis.set_roll_period(
             self, train_period=train_period, test_period=test_period,
-            start=start, end=end, epochs=epochs
+            start=start, end=end, repass_steps=repass_steps
         )
 
     def _train(self, X, y):
@@ -621,13 +359,13 @@ class RollMultiLayerPerceptron(MultiLayerPerceptron, _RollingBasis):
 
         return values
 
-    def unroll(self):
+    def unroll(self, display_slices = False):
+        self.handle_callback('begin_fit')
         for eval_slice, test_slice in self:
             # Compute prediction on eval and test set
-            self.y_eval[eval_slice] = self.sub_predict(self.X[eval_slice])
+            eval_prediction = self.sub_predict(self.X[eval_slice])
+            self.y_eval[eval_slice] = eval_prediction
             test_prediction = self.sub_predict(self.X[test_slice])
-            if abs(test_prediction.numpy().sum())<= 1e-6:
-                print('null prediction, investigate')
             self.y_test[test_slice] = test_prediction
 
             ev = self.y_eval[eval_slice]
@@ -644,15 +382,30 @@ class RollMultiLayerPerceptron(MultiLayerPerceptron, _RollingBasis):
             txt = '{:5.2%} is done | '.format(pct)
             txt += 'Eval loss is {:5.2} | '.format(self.loss_eval[-1])
             txt += 'Test loss is {:5.2} | '.format(self.loss_test[-1])
-            if np.random.rand()>=0.8:
+            if np.random.rand()>=0.9:
                 print(txt)
+            if display_slices:
+                print('eval slice ' + str(eval_slice))
+                print('test slice ' + str(test_slice))
+                if self.X[eval_slice].shape == self.X[test_slice].shape:
+                    m = (self.X[eval_slice] - self.X[test_slice])
+                    distinct_frobenius = (m * m).sum().sqrt()
+                    print('distinct frobenius between predictors ' + str(distinct_frobenius))
+                    t = test_prediction - eval_prediction
+                    distinct_frobenius_pred = (t * t).sum().sqrt()
+                    print('distinct frobenius between predictions ' + str(distinct_frobenius_pred))
+                if abs(test_prediction.numpy().sum()) <= 1e-6:
+                    print('null prediction, investigate')
+
+        self.last_eval_slice = eval_slice
+        self.last_test_slice = test_slice
+        return eval_slice, test_slice
 
     def unroll_features(self,dates, feature_names):
         rows_list = []
         dates_list = []
-
-
         for eval_slice, test_slice in self:
+            print()
             # Compute prediction on eval and test set
             self.y_eval[eval_slice] = self.sub_predict(self.X[eval_slice])
             test_prediction = self.sub_predict(self.X[test_slice])
@@ -679,7 +432,7 @@ class RollMultiLayerPerceptron(MultiLayerPerceptron, _RollingBasis):
             txt = '{:5.2%} is done | '.format(pct)
             txt += 'Eval loss is {:5.2} | '.format(self.loss_eval[-1])
             txt += 'Test loss is {:5.2} | '.format(self.loss_test[-1])
-            if np.random.rand()>=0.8:
+            if np.random.rand()>=0.9:
                 print(txt)
 
         features_df = pd.DataFrame(rows_list)
@@ -687,5 +440,37 @@ class RollMultiLayerPerceptron(MultiLayerPerceptron, _RollingBasis):
 
         return features_df
 
+    def print_parameters(self):
+        # Print model's state_dict
+        print("Model's state_dict:")
+        for param_tensor in self.state_dict():
+            print(param_tensor, "\t", self.state_dict()[param_tensor].size())
+
+        # Print optimizer's state_dict
+        print("Optimizer's state_dict:")
+        for var_name in self.optimizer.state_dict():
+            print(var_name, "\t", self.optimizer.state_dict()[var_name])
+
+    def save_model_and_last_state(self, state=None, model_path=''):
+
+        torch.save({
+            'model_state_dict': self.state_dict(),
+            'optimizer_state_dict': self.optimizer.state_dict()
+        }, model_path+'_model.torch')
+
+        file_obj = open(model_path+'_py.obj', 'wb')
+        pickle.dump(state, file_obj)
+        file_obj.close()
+
+
+    def resume_state(self, model_path=''):
+        checkpoint = torch.load(model_path+'_model.torch')
+        self.load_state_dict(checkpoint['model_state_dict'])
+        self.optimizer.load_state_dict(checkpoint['optimizer_state_dict'])
+        file_obj = open(model_path+'_py.obj', 'rb')
+        self.state = pickle.load(file_obj)
+        return self.state
+
+
 
 
