@@ -5,11 +5,11 @@ import sqlite3
 import pandas as pd
 
 from multiprocessing import Pool
-
+from napoleontoolbox.features import features_type
+from napoleontoolbox.file_saver import dropbox_file_saver
+from napoleontoolbox.neural_net import lrs_type
 class ParalleLauncher():
-
-
-    def __init__(self,  db_path, meta_stationarize=[True], meta_normalize=[True], meta_advance_features=[False], meta_advance_signals=[False], meta_history=[False], meta_layers=[[2048,1024,512,256,128,64,32]], convolutions=[0], activations=['relu'], meta_n=[126], meta_seeds=[0], epochss=[1], ss=[21], meta_n_past_features=[21], utilities=['f_drawdown']):
+    def __init__(self,drop_token='', local_root_directory='../data',user='napoleon',  db_path_suffix = '_run.sqlite', meta_features_types=[features_type.FeaturesType.STANDARD_ADVANCED], meta_layers=[[2048,1024,512,256,128,64,32]], convolutions=[0], activations=['relu'], meta_n=[126], meta_seeds=[0], epochss=[1], ss=[21], meta_n_past_features=[21], utilities=['f_drawdown'], meta_lrs_types = [lrs_type.LrType.CONSTANT],meta_lrs = [0.01]):
         # print('Epochs ' + str(epochss))
         # print('Seed ' + str(meta_seeds))
         # print('Training period size ' + str(meta_n))
@@ -28,31 +28,32 @@ class ParalleLauncher():
         # print('Rebalancing ' + str(ss))
         self.args = []
         self.counter = 1
-        for utility in utilities:
-            for n_past_features in meta_n_past_features:
-                for convolution in convolutions:
-                    for activation in activations:
-                        for s in ss:
-                            for epochs in epochss:
-                                for normalize in meta_normalize:
-                                    for stationarize in meta_stationarize:
-                                        for whole_history in meta_history:
-                                            for advance_feature in meta_advance_features:
-                                                for advance_signal in meta_advance_signals:
-                                                    for seed in meta_seeds:
-                                                        for layers in meta_layers:
-                                                            for n in meta_n:
-                                                                self.args.append((self,seed, utility, layers, epochs,
-                                                                             n_past_features, n, s, whole_history,
-                                                                             advance_feature, advance_signal, stationarize, normalize,
-                                                                             activation, convolution))
-                                                                self.counter = self.counter + 1
-
-        self.db_path = db_path
+        for lrs_type in meta_lrs_types:
+            for lr in meta_lrs:
+                for utility in utilities:
+                    for n_past_features in meta_n_past_features:
+                        for convolution in convolutions:
+                            for activation in activations:
+                                for s in ss:
+                                    for epochs in epochss:
+                                        for feature_type in meta_features_types:
+                                            for seed in meta_seeds:
+                                                for layers in meta_layers:
+                                                    for n in meta_n:
+                                                        self.args.append((self,seed, utility, layers, epochs,
+                                                                     n_past_features, n, s, feature_type,
+                                                                     activation, convolution,lr, lrs_type))
+                                                        self.counter = self.counter + 1
+        self.local_root_directory = local_root_directory
+        self.user = user
+        self.db_path_suffix = db_path_suffix
+        self.filename =  user + db_path_suffix
+        self.db_path = self.local_root_directory + self.filename
         self.runs = []
         self.totalRow = 0
         self.instantiateTable()
         self.empty_runs_to_investigate = []
+        self.dbx = dropbox_file_saver.NaPoleonDropboxConnector(drop_token)
 
     def launchParallelPool(self, toRun, use_num_cpu):
         with Pool(processes=use_num_cpu) as pool:
@@ -108,7 +109,6 @@ class ParalleLauncher():
             sqliteConnection = sqlite3.connect(self.db_path)
             cursor = sqliteConnection.cursor()
             for i, v in values.iteritems():
-                print("Connected to SQLite")
                 sqlite_insert_query = """INSERT INTO 'parallel_run' ('effective_date', '""" + run + """') VALUES ('""" + str(
                     i) + """','""" + str(v) + """');"""
                 cursor.execute(sqlite_insert_query)
@@ -129,7 +129,6 @@ class ParalleLauncher():
             sqliteConnection = sqlite3.connect(self.db_path)
             cursor = sqliteConnection.cursor()
             for i, v in values.iteritems():
-                print("Connected to SQLite")
                 sqlite_update_query = """UPDATE 'parallel_run' set '""" + run + """' = '""" + str(
                     v) + """' where effective_date = '""" + str(i) + """'"""
                 cursor.execute(sqlite_update_query)
@@ -148,8 +147,8 @@ class ParalleLauncher():
         try:
             table_name = run + '_weight'
             sqliteConnection = sqlite3.connect(self.db_path)
+            weights_df.columns = [col.replace(' ','_') for col in weights_df.columns]
             weights_df.to_sql(name=table_name, con=sqliteConnection)
-
         except sqlite3.Error as error:
             print("Error while creating a sqlite table", error)
         finally:
@@ -157,24 +156,39 @@ class ParalleLauncher():
                 sqliteConnection.close()
                 print("sqlite connection is closed")
 
+
+    def addRunRecordings(self,run, recordings_df):
+        sqliteConnection = None
+        try:
+            table_name = run + '_recording'
+            sqliteConnection = sqlite3.connect(self.db_path)
+            recordings_df.to_sql(name=table_name, con=sqliteConnection)
+        except sqlite3.Error as error:
+            print("Error while creating a sqlite table", error)
+        finally:
+            if (sqliteConnection):
+                sqliteConnection.close()
+                print("sqlite connection is closed")
+
+
     def addResults(self, run, values):
         success = self.insertResults(run,values)
         if not success :
             self.updateResults(run, values)
 
 
-    def saveResults(self, run, values, weights_df):
+    def saveResults(self, run, values, weights_df, recordings_df):
         self.addRun(run)
         self.addResults(run, values)
         self.totalRow = self.datesCount()
         self.addRunWeights(run, weights_df)
+        self.addRunRecordings(run, recordings_df)
 
     def datesCount(self):
         sqliteConnection = None
         try:
             sqliteConnection = sqlite3.connect(self.db_path, timeout=20)
             cursor = sqliteConnection.cursor()
-            print("Connected to SQLite")
             sqlite_select_query = """SELECT count(*) from parallel_run"""
             cursor.execute(sqlite_select_query)
             totalRows = cursor.fetchone()
@@ -189,84 +203,7 @@ class ParalleLauncher():
                 print("The Sqlite connection is closed")
 
 
-    def getAllRuns(self):
-        sqliteConnection = None
-        try:
-            sqliteConnection = sqlite3.connect(self.db_path, timeout=20)
-            cursor = sqliteConnection.cursor()
-            print("Connected to SQLite")
-            sqlite_select_runs =  """PRAGMA table_info(parallel_run);"""
-            cursor.execute(sqlite_select_runs)
-            all_runs_tuple = cursor.fetchall()
-
-            cursor.close()
-            all_runs = [r[1] for r in all_runs_tuple if not 'date' in r[1]]
-            print("Total run numbers :  ", len(all_runs))
-
-            return all_runs
-        except sqlite3.Error as error:
-            print("Failed to read data from sqlite table", error)
-        finally:
-            if (sqliteConnection):
-                sqliteConnection.close()
-                print("The Sqlite connection is closed")
-
-    def analyzeRunResults(self, run):
-        runs = self.getAllRuns()
-        results_df = None
-
-        if run not in runs :
-            return results_df
-
-        sqliteConnection = None
-
-        try:
-            sqliteConnection = sqlite3.connect(self.db_path, timeout=20)
-            sqlite_select_run_query = """SELECT effective_date, """+run+""" from parallel_run order by effective_date asc"""
-            results_df = pd.read_sql_query(sqlite_select_run_query, sqliteConnection)
-        except sqlite3.Error as error:
-            print("Failed to read data from sqlite table", error)
-        finally:
-            if (sqliteConnection):
-                sqliteConnection.close()
-                print("The Sqlite connection is closed")
-        return results_df
-
-    def analyzeAllRunResults(self):
-        runs = self.getAllRuns()
-        if len(runs)>1:
-            runs = ','.join(runs)
-        else :
-            runs = runs[0]
-
-        sqliteConnection = None
-
-        try:
-            sqliteConnection = sqlite3.connect(self.db_path, timeout=20)
-            sqlite_select_run_query = """SELECT effective_date, """+runs+""" from parallel_run order by effective_date asc"""
-            results_df = pd.read_sql_query(sqlite_select_run_query, sqliteConnection)
-        except sqlite3.Error as error:
-            print("Failed to read data from sqlite table", error)
-        finally:
-            if (sqliteConnection):
-                sqliteConnection.close()
-                print("The Sqlite connection is closed")
-
-
-        results_df = results_df.rename(columns={"effective_date": "Date"})
-        results_df['Date'] = pd.to_datetime(results_df['Date'])
-        results_df = results_df.sort_values(by=['Date'])
-        results_df = results_df.set_index(results_df['Date'])
-        results_df = results_df.drop(['Date'], axis=1)
-
-
-        run_empty_results = results_df.sum(axis = 0)
-        self.empty_runs_to_investigate = list(run_empty_results.index[run_empty_results == 0])
-
-        results_df = results_df.drop(columns=self.empty_runs_to_investigate)
-        return results_df
-
-
-
-
 
+    def upload_results_to_dropbox(self):
+        print('uploading to dropbox the results '+self.db_path)
+        self.dbx.uploadFileToDropbox(filename=self.filename,fullpath = self.db_path)
